﻿using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(AudioSource))]
public class AudioManager : MonoBehaviour
{
    public static AudioManager instance;

    private AudioSource mainSource;

    [Header("Music")]
    public AudioSource musicSource;
    public AudioClip gameMusicClip;
    public AudioClip gameOverClip;
    public AudioClip menuClip;

    [Header("Player sound effects")]
    public AudioClip duckDive;
    public AudioClip duckHop;
    public AudioClip duckJump;
    public AudioClip duckSplash;
    
    [Header("Environmental SFX")]
    public AudioClip bubbleBurst;
    public AudioClip lavaDeath;
    public AudioClip floorDeath;
    public AudioClip lavaWave;

    [Header("UI & Other Sound effects")]
    public AudioClip duckBoing;
    public AudioClip duckRocketJump;

    [Header("StartMenu Quaks")]
    public AudioClip[] quaks;

    void Awake()
    {
        if (instance == null) { instance = this; }
        else if (instance != this) { Destroy(gameObject); }
        DontDestroyOnLoad(gameObject);

        // center this manager, so if we play around with spatial audio we at least have the sound centered
        transform.position = new Vector3(Screen.width / 2, Screen.height / 2);
    }

    void Start()
    {
        mainSource = GetComponent<AudioSource>();
        AudioManager.PlayMusic(AudioManager.instance.menuClip, true);
    }
    
    public static void PlayClip(AudioClip clip, bool loop = false)
    {
        instance.mainSource.PlayOneShot(clip);
    }

    public static void PlayMusic(AudioClip clip, bool loop = false)
    {
        instance.musicSource.clip = clip;
        instance.musicSource.loop = loop;
        instance.musicSource.Play();        
    }

    public static void StopMusic()
    {
        instance.musicSource.Stop();
    }
}