﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CameraController : MonoBehaviour
{
    public static CameraController instance;

    [SerializeField]
    private float minSize = 10;

    [SerializeField]
    private float maxSize = 25;

    [SerializeField]
    private float minCamSpeed = 3.5f;

    private Vector3 shake;

    private List<Transform> transforms = new List<Transform>();

    private void Awake()
    {
        if (instance == null) { instance = this; }
        else if (instance != this) { Destroy(gameObject); }
        DontDestroyOnLoad(gameObject);
    }

    void Start()
    {
        Camera camera = Camera.main;
    }

    void Update()
    {
        UpdateTransforms();

        if (transforms.Count > 0)
        {
            float dist = CalculateBounds(transforms);
            CalculateCameraPosAndSize(transforms, dist);
        }
        Camera.main.transform.position += shake;
    }

    public void UpdateTransforms()
    {
        transforms = new List<Transform>();

        // we include the objects generated by wave spawner, so they will be on screen as well
        transforms.AddRange(WaveSpawner.instance.GetActiveWaves());

        // and of course add players
        for (int i = 0; i < GameManager.instance.playerControllers.Count; ++i)
        {
            if ((GameManager.instance.playerControllers[i].state != PlayerController.State.Dead))
            {
                transforms.Add(GameManager.instance.playerControllers[i].transform);
            }
        }
    }

    private float CalculateBounds(List<Transform> transforms)
    {
        float maxDist = 0;
        for (int i = 0; i < transforms.Count - 1; ++i)
        {
            Transform ti = transforms[i];
            for (int j = i + 1; j < transforms.Count; ++j)
            {
                Transform tj = transforms[j];
                float dist = Vector3.Distance(ti.position, tj.position);
                if (dist > maxDist)
                    maxDist = dist;
            }
        }
        return maxDist;
    }

    private void CalculateCameraPosAndSize(List<Transform> transforms, float dist)
    {
        Vector3 cameraCenter = Vector3.zero;
        foreach (Transform t in transforms)
        {
            cameraCenter += t.position;
        }
        Vector3 finalCameraCenter = cameraCenter / transforms.Count;
        finalCameraCenter.z = -30;

        Vector2 pos = Camera.main.transform.position;
        Vector2 desiredPos = finalCameraCenter;

        if (Vector2.Distance(pos, desiredPos) > minCamSpeed)
        {
            Camera.main.transform.position = Vector3.Lerp(Camera.main.transform.position, finalCameraCenter, Time.deltaTime);
        }
        Camera.main.orthographicSize = Mathf.Lerp(Camera.main.orthographicSize, Mathf.Clamp(dist * .7f, minSize, maxSize), Time.deltaTime);
    }

    public IEnumerator Shake(float duration, float magnitude)
    {
        float elapsed = 0.0f;

        while (elapsed < duration)
        {

            elapsed += Time.deltaTime;

            float percentComplete = elapsed / duration;
            float damper = 1.0f - Mathf.Clamp(4.0f * percentComplete - 3.0f, 0.0f, 1.0f);

            // map value to [-1, 1]
            float x = Random.value * 2.0f - 1.0f;
            float y = Random.value * 2.0f - 1.0f;
            x *= magnitude * damper;
            y *= magnitude * damper;

            shake = new Vector3(x, y, 0);

            yield return null;
        }
        shake = Vector3.zero;
    }
}
