﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class UIManager : MonoBehaviour
{
    public static UIManager instance;

    public enum UIState { MAIN, READY, RESULTS }
    public static UIState currentState;

    [Header("Prefabs")]
    public GameObject playerSlot;

    [Header("Scene ref")]
    public GameObject mainPanel;
    public GameObject readyPanel;
    public GameObject resultsPanel;
    public Image resultsPanelImage;

    public GameObject playerSlotContainer;

    public Text timeText;
    public Text resultsText;

    private static List<GameObject> playerSlots = new List<GameObject>();

    private void Awake()
    {
        if (instance == null) { instance = this; }
        else if (instance != this) { Destroy(gameObject); }
        DontDestroyOnLoad(gameObject);
    }

    public static void SwitchState(UIState state)
    {
        currentState = state;

        instance.mainPanel.SetActive(false);
        instance.readyPanel.SetActive(false);
        instance.resultsPanel.SetActive(false);

        switch (currentState)
        {
            case UIState.MAIN:
                instance.mainPanel.SetActive(true);
                break;
            case UIState.READY:
                for (int i = 0; i < playerSlots.Count; i++) { Destroy(playerSlots[i]); }
                playerSlots = new List<GameObject>();
                instance.readyPanel.SetActive(true);
                break;
            case UIState.RESULTS:
                instance.resultsPanel.SetActive(true);
                break;
            default:
                break;
        }
    }

    public void AddPlayer()
    {
        // nog magic numbers :') we get/calculate data from the prefab
        RectTransform originalRT = instance.playerSlot.GetComponent<RectTransform>();
        float slotWidth = originalRT.anchorMax.x - originalRT.anchorMin.x;

        // put object into the scene and reset its transform
        GameObject goInstance = Instantiate(instance.playerSlot, instance.playerSlotContainer.transform);

        StartCoroutine(blaat(goInstance, playerSlots.Count));

        RectTransform rt = goInstance.GetComponent<RectTransform>();
        rt.anchoredPosition = Vector2.zero;
        rt.sizeDelta = Vector2.zero;

        // let's keep track of it in a list
        playerSlots.Add(goInstance);

        // now we do some magic to reposition the already existing objects
        bool evenNumber = playerSlots.Count % 2 == 0;
        float startPos = 0.5f;
        if (!evenNumber || playerSlots.Count == 1) { startPos -= slotWidth / 2.0f; }
        //startPos -= playerSlots.Count * slotWidth;

        for (int i = 0; i < playerSlots.Count; i++)
        {
            rt = playerSlots[i].GetComponent<RectTransform>();
            float pos = startPos + i * slotWidth;
            rt.anchorMin = new Vector2(pos, originalRT.anchorMin.y);
            rt.anchorMax = new Vector2(pos + slotWidth, originalRT.anchorMax.y);
        }
    }

    private IEnumerator blaat(GameObject goInstance, int index)
    {
        yield return null;
        goInstance.transform.GetChild(0).GetComponentInChildren<Image>().sprite = LifesManager.instance.counterSprites[GameManager.instance.GetSkinIndex(index)];
    }

    public static void TogglePlayerReady(int index, bool ready)
    {
        playerSlots[index].GetComponentInChildren<Text>().text = ready ? "READY!!!" : "No...";
    }

    public void AdjustResultsText(PlayerController pc)
    {
        resultsPanelImage.sprite = LifesManager.instance.counterSprites[GameManager.instance.GetSkinIndex(pc)];
        instance.resultsText.text = string.Format("A winner is you!\nController {0}", pc.playerJoypadNumber);
    }
}