﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class WaveSpawner : MonoBehaviour
{
    public static WaveSpawner instance;

    [SerializeField]
    private float interval;

    [SerializeField]
    private float alertWaveSpeed;

    [SerializeField]
    private float alertWaveInterval;

    [SerializeField]
    private float speed;

    [SerializeField]
    private GameObject smallWave;

    [SerializeField]
    private GameObject mediumWave;

    [SerializeField]
    private GameObject largeWave;

    [SerializeField]
    private CameraController cameraController;

    private void Awake()
    {
        if (instance == null) { instance = this; }
        else if (instance != this) { Destroy(gameObject); }
        DontDestroyOnLoad(gameObject);
    }

    public void StartSpawningWaves()
    {
        StartCoroutine("SpawnWave");
    }

    public void StopSpawningWaves()
    {
        StopCoroutine("SpawnWave");
    }

    void Update()
    {

    }

    public List<Transform> GetActiveWaves()
    {
        List<Transform> transforms = new List<Transform>();
        for (int i = 0; i < this.gameObject.transform.childCount; i++)
        {
            Transform trans = this.gameObject.transform.GetChild(i);
            if (trans.position.x > -20)
                transforms.Add(trans);
        }
        return transforms;
    }

    private IEnumerator SpawnWave()
    {
        while (true)
        {


            yield return new WaitForSeconds(this.interval);

            AudioManager.PlayClip(AudioManager.instance.lavaWave);

            yield return new WaitForSeconds(1f);

            GameObject.Instantiate(smallWave, this.transform);
            yield return new WaitForSeconds(this.alertWaveInterval);

            StartCoroutine(cameraController.Shake(8, .25f));

            AudioManager.PlayClip(AudioManager.instance.lavaWave);
            AudioManager.PlayClip(AudioManager.instance.lavaWave);

            GameObject.Instantiate(mediumWave, this.transform);
            yield return new WaitForSeconds(this.alertWaveInterval);

            StartCoroutine(cameraController.Shake(4, .5f));

            AudioManager.PlayClip(AudioManager.instance.lavaWave);
            AudioManager.PlayClip(AudioManager.instance.lavaWave);
            AudioManager.PlayClip(AudioManager.instance.lavaWave);

            yield return new WaitForSeconds(2);
            GameObject.Instantiate(largeWave, this.transform);
        }
    }

}
